import { F, contains, curry, cond, when, pipe, nthArg, is, T, always, __ } from 'ramda'
import isPatch from './isPatch'

const valueType = cond([
  [ contains(__, ['move', 'copy']), always('from') ],
  [ contains(__, ['add', 'replace', 'test', 'merge']), always('value') ],
  [ T, F ]
])

/**
 * Creates a JSON Patches given an operator, path and a value
 *
 * https://tools.ietf.org/html/rfc6902
 *
 * https://tools.ietf.org/html/rfc7386
 *
 * ** Sig: String -> String -> A -> Object **
 *
 * @function
 * @implements curry
 * @param {string} operation - A JSON Patch operation (see https://tools.ietf.org/html/rfc6902)
 * @param {string} pointer - A JSON Pointer (see http://tools.ietf.org/html/rfc6901)
 * @param {object} value|from path - A value for [add, replace, test] and a FROM path for [move, copy]
 * @return {object} jsonPatch - A valid json patch
 * @example
 *
 * lib.createPatch('add', '/foo/bar', 123) // { op: 'add', path: '/foo/bar', value: 123 }
 *
 */
const createPatch = curry((op, jsonPath, value) => {

  let patch = {
    op: op,
    path: jsonPath
  }

  if (valueType(op) !== false) {
    patch[valueType(op)] = value
  }

  let result = isPatch(patch)

  if (true !== result) {
    return new Error(result)
  }

  return patch
})

export default createPatch
